@@ -17,6 +17,7 @@ import random
 import hashlib
 import logging
 import threading
+import traceback
 from collections import OrderedDict
 try:
     from urllib.parse import urlparse, urlencode, urlunparse
@@ -27,8 +28,8 @@ except ImportError:
     from urllib import urlencode
     from urllib2 import urlopen, Request, HTTPError, URLError
 
-import artella
-from artella.core import consts, utils, exceptions
+from artella import dcc
+from artella.core import consts, utils, exceptions, dccplugin
 
 logger = logging.getLogger('artella')
 
@@ -119,8 +120,9 @@ class ArtellaDriveClient(object):
         """
         Checks whether or not current client is available and running
 
-        :param update:
-        :return:
+        :param update: bool, Whether or not remote sessions should be updated
+        :return: True if the client is available and running; False otherwise.
+        :rtype: bool
         """
 
         return self._available and self._running and self.get_remote_sessions(update=update)
@@ -200,8 +202,6 @@ class ArtellaDriveClient(object):
         :rtype: list(str)
         """
 
-        from artella import dcc
-
         def _ping_remote_sessions():
             rsp = self.ping()
             if 'error' in rsp:
@@ -512,7 +512,7 @@ class ArtellaDriveClient(object):
         path = utils.clean_path(os.path.expanduser(os.path.expandvars(path)))
 
         local_root = self.get_local_root()
-        local_project_names = (self.get_local_projects() or dict()).keys()
+        local_project_names = list((self.get_local_projects() or dict()).keys())
 
         for old_alr in consts.OLD_LOCAL_ROOTS:
             old_alr_str = '${}'.format(old_alr)
@@ -623,7 +623,7 @@ class ArtellaDriveClient(object):
         :rtype: str
         """
 
-        local_project_names = (self.get_local_projects() or dict()).keys()
+        local_project_names = list((self.get_local_projects() or dict()).keys())
 
         project_name = None
 
@@ -654,7 +654,7 @@ class ArtellaDriveClient(object):
             logger.warning('Was not possible to retrieve Project ID from path: {}!'.format(file_path))
             return None
 
-        if project_name not in local_projects.keys():
+        if project_name not in list(local_projects.keys()):
             logger.warning(
                 'Was not possible to retrieve Project ID from path because project "{}" is not recognized!'.format(
                     project_name))
@@ -762,7 +762,7 @@ class ArtellaDriveClient(object):
         """
 
         path_handle_map = paths_to_handles(paths, as_dict=True)
-        handles = path_handle_map.values()
+        handles = list(path_handle_map.values())
         if not path_handle_map or not handles:
             return dict()
         logger.debug('Handles: "{}"'.format(handles))
@@ -775,11 +775,14 @@ class ArtellaDriveClient(object):
         }
         if version is not None:
             payload['version'] = int(version)
-        req = Request('http://{}:{}/v2/localserve/transfer/download'.format(self._host, self._port))
-        rsp = self._communicate(req, json.dumps(payload).encode())
+        try:
+            req = Request('http://{}:{}/v2/localserve/transfer/download'.format(self._host, self._port))
+            rsp = self._communicate(req, json.dumps(payload).encode())
+        except Exception:
+            rsp = {'error': traceback.format_exc()}
         if 'error' in rsp:
             logger.warning(
-                'Unable to download file paths "{}" "{}"'.format(path_handle_map.keys(), rsp.get('error')))
+                'Unable to download file paths "{}" "{}"'.format(list(path_handle_map.keys()), rsp.get('error')))
             return rsp
 
         return self._track_response(rsp)
@@ -874,7 +877,7 @@ class ArtellaDriveClient(object):
         rsp = self._communicate(req, json.dumps(payload).encode())
         if 'error' in rsp:
             logger.warning(
-                'Unable to upload file paths "{}" "{}"'.format(path_handle_map.keys(), rsp.get('error')))
+                'Unable to upload file paths "{}" "{}"'.format(list(path_handle_map.keys()), rsp.get('error')))
             return rsp
 
         return self._track_response(rsp)
@@ -953,6 +956,7 @@ class ArtellaDriveClient(object):
         Returns current version of the given file
 
         :param str file_path: Absolute local file path to retrieve current local version of
+        :param str _status: new file status
         :return: Current local version of the given file path
         :rtype: int
         """
@@ -1080,7 +1084,9 @@ class ArtellaDriveClient(object):
         tx_bytes_total = 0
         batches_complete = set()
 
-        for batch_id in self._batch_ids:
+        batch_ids = list(self._batch_ids)[:]
+
+        for batch_id in batch_ids:
             params = urlencode({'batch-id': batch_id, 'details': True})
             req = Request('http://{}:{}/v2/localserve/progress/summary?{}'.format(self._host, self._port, params))
             rsp = self._communicate(req)
@@ -1327,7 +1333,7 @@ class ArtellaDriveClient(object):
 
             self._available = True
             logger.debug('Message received: {}'.format(msg))
-            artella.DccPlugin().pass_message(msg)
+            dccplugin.DccPlugin().pass_message(msg)
 
         # If Artella Drive Client is nor running we force disconnection
         # self._disconnect()
