@@ -12,10 +12,9 @@ import time
 import logging
 import threading
 
-import artella
-import artella.dcc as dcc
-import artella.register as register
-from artella.core import utils, qtutils, splash, client
+from artella import dcc
+from artella.core.dcc import parser
+from artella.core import utils, qtutils, callbacks, splash
 from artella.widgets import snackbar
 
 if qtutils.QT_AVAILABLE:
@@ -23,14 +22,36 @@ if qtutils.QT_AVAILABLE:
 
 logger = logging.getLogger('artella')
 
+_DCC_PLUGIN = None
 
-class ArtellaDccPlugin(object):
+
+class _MetaDccPlugin(type):
+
+    def __call__(cls, *args, **kwargs):
+
+        global _DCC_PLUGIN
+        if _DCC_PLUGIN:
+            return _DCC_PLUGIN
+
+        if dcc.is_maya():
+            from artella.dccs.maya import plugin
+            _DCC_PLUGIN = type.__call__(plugin.ArtellaMayaPlugin, *args, **kwargs)
+        elif dcc.is_unreal():
+            from artella.dccs.unreal import plugin
+            _DCC_PLUGIN = type.__call__(plugin.ArtellaUnrealPlugin, *args, **kwargs)
+        else:
+            _DCC_PLUGIN = type.__call__(BaseArtellaDccPlugin, *args, **kwargs)
+
+        return _DCC_PLUGIN
+
+
+class BaseArtellaDccPlugin(object):
 
     MENU_NAME = 'Artella'
     _ASYNC_INVOKER, _SYNC_INVOKER = range(2)
 
     def __init__(self, artella_drive_client):
-        super(ArtellaDccPlugin, self).__init__()
+        super(BaseArtellaDccPlugin, self).__init__()
 
         self._artella_drive_client = artella_drive_client
         self._dev = False
@@ -91,7 +112,7 @@ class ArtellaDccPlugin(object):
 
         return plugin_version
 
-    def init(self, dev=False, show_dialogs=True, create_menu=True, create_callbacks=True, *args, **kwargs):
+    def init(self, dev=False, show_dialogs=True, create_menu=True, create_callbacks=True,  *args, **kwargs):
         """
         Initializes Artella plugin in current DCC.
 
@@ -117,7 +138,9 @@ class ArtellaDccPlugin(object):
             dcc.execute_deferred(self.create_menus)
 
         # Initialize Artella Drive client
-        self.init_client(show_dialogs=show_dialogs)
+        _init_client = kwargs.pop('init_client', True)
+        if _init_client:
+            self.init_client(show_dialogs=show_dialogs)
 
         logger.debug('trying to create quit signal ...')
         if qtutils.QT_AVAILABLE:
@@ -143,8 +166,8 @@ class ArtellaDccPlugin(object):
 
         artella_drive_client = self._artella_drive_client or self.get_client(show_dialogs=show_dialogs)
         if artella_drive_client:
-            artella_drive_client.artella_drive_listen()
             self.setup_project(artella_drive_client.get_local_root())
+            artella_drive_client.artella_drive_listen()
         else:
             logger.warning(
                 'Artella Drive Client was not initialized. Artella server '
@@ -160,9 +183,6 @@ class ArtellaDccPlugin(object):
         :rtype: bool
         """
 
-        if not self._artella_drive_client:
-            return False
-
         # Remove Artella callbacks
         self.remove_callbacks()
 
@@ -172,6 +192,9 @@ class ArtellaDccPlugin(object):
         else:
             dcc.execute_deferred(self.remove_menus)
 
+        if not self._artella_drive_client:
+            return False
+
         self._artella_drive_client.artella_drive_disconnect()
         self._artella_drive_client = None
 
@@ -265,9 +288,7 @@ class ArtellaDccPlugin(object):
         :return:
         """
 
-        import artella.core.callback as callback
-
-        callback.initialize_callbacks()
+        callbacks.initialize_callbacks()
 
     def remove_callbacks(self):
         """
@@ -275,9 +296,7 @@ class ArtellaDccPlugin(object):
         :return:
         """
 
-        import artella.core.callback as callback
-
-        callback.uninitialize_callbacks()
+        callbacks.uninitialize_callbacks()
 
     def validate_environment_for_callback(self, callback_name):
         """
@@ -317,6 +336,9 @@ class ArtellaDccPlugin(object):
         :rtype: ArtellaDriveClient or None
         """
 
+        # To avoid cyclic imports
+        from artella.core import client
+
         if not self._artella_drive_client:
             # TODO: Here we are not taking into account custom extensions (check loader). We should store them
             # TODO: in an env variable and access them here
@@ -411,6 +433,7 @@ class ArtellaDccPlugin(object):
         Returns whether or not given file path is an Artella file path or not
         A path is considered to be an Artella path if the path is located inside the Artella project folder
         in the user machine
+
         :param str file_path: path to check. If not given, current DCC scene file path will be used
         :return: True if the given file path is an Artella path; False otherwise.
         :rtype: bool
@@ -504,6 +527,7 @@ class ArtellaDccPlugin(object):
     def update_paths(self, file_path=None, show_dialogs=True, call_post_function=True, skip_save=True):
         """
         Updates all file paths of the given file path to make sure that they point to valid Artella file paths
+
         :param str file_path:
         :param bool show_dialogs:
         :param bool call_post_function:
@@ -536,7 +560,7 @@ class ArtellaDccPlugin(object):
                 logger.info('Skipping non DCC scene file path from conversion: "{}"'.format(local_path))
                 continue
 
-            can_lock = artella.DccPlugin().can_lock_file(local_path, show_dialogs=False)
+            can_lock = self.can_lock_file(local_path, show_dialogs=False)
             if not can_lock:
                 logger.warning(
                     'File "{}" cannot be locked and paths cannot be updated. Skipping ...'.format(local_path))
@@ -545,8 +569,8 @@ class ArtellaDccPlugin(object):
             if dcc.scene_name() != file_path:
                 dcc.open_scene(file_path, save=True)
 
-            parser = artella.Parser()
-            valid_convert, updated_paths = parser.update_paths(local_path)
+            dcc_parser = parser.Parser()
+            valid_convert, updated_paths = dcc_parser.update_paths(local_path)
             updated_paths = utils.force_list(updated_paths)
             converted_paths.extend(updated_paths)
 
@@ -898,12 +922,12 @@ class ArtellaDccPlugin(object):
 
     def show_artella_message(self, text, title='', duration=None, closable=True):
         """
-        Shows an artella message
-        :param text:
-        :param title:
-        :param duration:
-        :param closable:
-        :return:
+        Shows an Artella message
+
+        :param text: str, artella text to show
+        :param title: str, title of the artella message
+        :param duration: float or None, if given, message only will appear the specified seconds
+        :param closable: bool, Whether the message can be closed by the user or not (when duration is given)
         """
 
         logger.debug(str(text))
@@ -915,11 +939,11 @@ class ArtellaDccPlugin(object):
     def show_success_message(self, text, title='', duration=None, closable=True):
         """
         Shows a success message
-        :param text:
-        :param title:
-        :param duration:
-        :param closable:
-        :return:
+
+        :param text: str, success text to show
+        :param title: str, title of the success message
+        :param duration: float or None, if given, message only will appear the specified seconds
+        :param closable: bool, Whether the message can be closed by the user or not (when duration is given)
         """
 
         logger.info(str(text))
@@ -931,11 +955,11 @@ class ArtellaDccPlugin(object):
     def show_info_message(self, text, title='', duration=None, closable=True):
         """
         Shows an info message
-        :param text:
-        :param title:
-        :param duration:
-        :param closable:
-        :return:
+
+        :param text: str, info text to show
+        :param title: str, title of the info message
+        :param duration: float or None, if given, message only will appear the specified seconds
+        :param closable: bool, Whether the message can be closed by the user or not (when duration is given)
         """
 
         logger.info(str(text))
@@ -947,11 +971,11 @@ class ArtellaDccPlugin(object):
     def show_warning_message(self, text, title='', duration=None, closable=True):
         """
         Shows a warning message
-        :param text:
-        :param title:
-        :param duration:
-        :param closable:
-        :return:
+
+        :param text: str, warning text to show
+        :param title: str, title of the warning message
+        :param duration: float or None, if given, message only will appear the specified seconds
+        :param closable: bool, Whether the message can be closed by the user or not (when duration is given)
         """
 
         logger.warning(str(text))
@@ -963,10 +987,11 @@ class ArtellaDccPlugin(object):
     def show_error_message(self, text, title='', duration=None, closable=True):
         """
         Shows an error message
-        :param text:
-        :param title:
-        :param duration:
-        :param closable:
+
+        :param text: str, error text to show
+        :param title: str, title of the error message
+        :param duration: float or None, if given, message only will appear the specified seconds
+        :param closable: bool, Whether the message can be closed by the user or not (when duration is given)
         :return:
         """
 
@@ -1040,7 +1065,8 @@ class ArtellaDccPlugin(object):
         if dcc_main_thread_fn:
             return dcc_main_thread_fn(fn, *args, **kwargs)
         else:
-            invoker = (self._main_thread_invoker if invoker_id == self._SYNC_INVOKER else self._main_thread_async_invoker)
+            invoker = (
+                self._main_thread_invoker if invoker_id == self._SYNC_INVOKER else self._main_thread_async_invoker)
             if invoker:
                 if QtWidgets.QApplication.instance() and (
                         QtCore.QThread.currentThread() != QtWidgets.QApplication.instance().thread()):
@@ -1086,7 +1112,7 @@ class ArtellaDccPlugin(object):
                     """
 
                     # Acquire lock to make sure that both function and result are not overwritten by synchronous calls
-                    # to this method from differnt threads
+                    # to this method from different threads
                     self._lock.acquire()
 
                     try:
@@ -1102,6 +1128,7 @@ class ArtellaDccPlugin(object):
                     finally:
                         self._lock.release()
 
+                @QtCore.Slot()
                 def _do_invoke(self):
                     """
                     Internal function that executes the function
@@ -1184,10 +1211,6 @@ class ArtellaDccPlugin(object):
         pass
 
 
-@utils.Singleton
-class ArtellaDccPluginSingleton(ArtellaDccPlugin, object):
-    def __init__(self, artella_drive_client=None):
-        ArtellaDccPlugin.__init__(self, artella_drive_client=artella_drive_client)
-
-
-register.register_class('DccPlugin', ArtellaDccPluginSingleton)
+@utils.add_metaclass(_MetaDccPlugin)
+class DccPlugin(object):
+    pass
